@@ -20,12 +20,14 @@
 //! 2.0 license.
 
 #![cfg_attr(not(feature = "std"), no_std)]
-#![cfg_attr(not(feature = "std"), feature(alloc))]
 
-#[cfg(not(feature = "std"))]
+#[cfg(all(test, not(feature = "std")))]
 #[macro_use]
 extern crate alloc;
 
+#[cfg(all(not(test), not(feature = "std")))]
+extern crate alloc;
+
 #[cfg(not(feature = "std"))]
 use alloc::vec::Vec;
 
@@ -39,6 +41,9 @@ pub(crate) mod lib {
 
     #[cfg(not(feature = "std"))]
     pub(crate) use core::*;
+
+    #[cfg(all(test, not(feature = "std")))]
+    pub(crate) use alloc::{boxed, rc, borrow};
 }
 
 use lib::borrow::{Borrow, BorrowMut};
@@ -131,7 +136,6 @@ impl_array! { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 20, 24,
 /// let mut stack_vec = StackVec::<[u8; 8]>::new();
 /// initialize(&mut stack_vec);
 /// ```
-#[deprecated(note = "Use `Extend` and `Deref<[T]>` instead")]
 pub trait VecLike<T>:
         ops::Index<usize, Output=T> +
         ops::IndexMut<usize> +
@@ -293,19 +297,17 @@ pub struct StackVec<A: Array> {
     // The capacity field is used for iteration and other optimizations.
     // Publicly expose the fields, so they may be used in constant
     // initialization.
+    pub data: mem::MaybeUninit<A>,
     pub length: usize,
-    pub data: mem::ManuallyDrop<A>,
 }
 
 impl<A: Array> StackVec<A> {
     /// Construct an empty vector
     #[inline]
     pub fn new() -> StackVec<A> {
-        unsafe {
-            StackVec {
-                length: 0,
-                data: mem::uninitialized(),
-            }
+        StackVec {
+            length: 0,
+            data: mem::MaybeUninit::uninit(),
         }
     }
 
@@ -328,6 +330,7 @@ impl<A: Array> StackVec<A> {
     }
 
     /// Construct a new `StackVec` from a `Vec<A::Item>` without bounds checking.
+    #[allow(deprecated)]
     pub unsafe fn from_vec_unchecked(mut vec: Vec<A::Item>) -> StackVec<A> {
         let mut data: A = mem::uninitialized();
         let len = vec.len();
@@ -336,7 +339,7 @@ impl<A: Array> StackVec<A> {
 
         StackVec {
             length: len,
-            data: mem::ManuallyDrop::new(data),
+            data: mem::MaybeUninit::new(data),
         }
     }
 
@@ -355,7 +358,7 @@ impl<A: Array> StackVec<A> {
     pub fn from_buf(buf: A) -> StackVec<A> {
         StackVec {
             length: A::size(),
-            data: mem::ManuallyDrop::new(buf),
+            data: mem::MaybeUninit::new(buf),
         }
     }
 
@@ -395,7 +398,7 @@ impl<A: Array> StackVec<A> {
     pub unsafe fn from_buf_and_len_unchecked(buf: A, len: usize) -> StackVec<A> {
         StackVec {
             length: len,
-            data: mem::ManuallyDrop::new(buf),
+            data: mem::MaybeUninit::new(buf),
         }
     }
 
@@ -547,7 +550,7 @@ impl<A: Array> StackVec<A> {
         }
 
         let (lower_size_bound, _) = iter.size_hint();
-        assert!(lower_size_bound <= std::isize::MAX as usize);  // Ensure offset is indexable
+        assert!(lower_size_bound <= lib::isize::MAX as usize);  // Ensure offset is indexable
         assert!(index + lower_size_bound >= index);             // Protect against overflow
         assert!(self.len() + lower_size_bound <= self.capacity());
 
@@ -595,9 +598,9 @@ impl<A: Array> StackVec<A> {
             Err(self)
         } else {
             unsafe {
-                let data = ptr::read(&self.data);
-                mem::forget(self);
-                Ok(mem::ManuallyDrop::into_inner(data))
+                let this = mem::ManuallyDrop::new(self);
+                let array = ptr::read(this.as_ptr() as *const A);
+                Ok(array)
             }
         }
     }
@@ -674,9 +677,10 @@ impl<A: Array> StackVec<A> where A::Item: Copy {
         StackVec {
             length: slice.len(),
             data: unsafe {
-                let mut data: A = mem::uninitialized();
-                ptr::copy_nonoverlapping(slice.as_ptr(), data.ptr_mut(), slice.len());
-                mem::ManuallyDrop::new(data)
+                let mut data: mem::MaybeUninit<A> = mem::MaybeUninit::uninit();
+                let ptr = data.as_mut_ptr() as *mut A::Item;
+                ptr::copy_nonoverlapping(slice.as_ptr(), ptr, slice.len());
+                data
             }
         }
     }
@@ -751,7 +755,8 @@ impl<A: Array> ops::Deref for StackVec<A> {
     #[inline]
     fn deref(&self) -> &[A::Item] {
         unsafe {
-            slice::from_raw_parts(self.data.ptr(), self.len())
+            let ptr = self.data.as_ptr() as *const A::Item;
+            slice::from_raw_parts(ptr, self.len())
         }
     }
 }
@@ -760,7 +765,8 @@ impl<A: Array> ops::DerefMut for StackVec<A> {
     #[inline]
     fn deref_mut(&mut self) -> &mut [A::Item] {
         unsafe {
-            slice::from_raw_parts_mut(self.data.ptr_mut(), self.len())
+            let ptr = self.data.as_mut_ptr() as *mut A::Item;
+            slice::from_raw_parts_mut(ptr, self.len())
         }
     }
 }
@@ -871,7 +877,6 @@ impl<A: Array> ExtendFromSlice<A::Item> for StackVec<A> where A::Item: Copy {
     }
 }
 
-#[allow(deprecated)]
 impl<A: Array> VecLike<A::Item> for StackVec<A> {
     #[inline]
     fn push(&mut self, value: A::Item) {
@@ -1131,8 +1136,10 @@ macro_rules! stackvec {
 #[cfg(test)]
 mod test {
     use super::*;
-    use super::lib::iter::FromIterator;
-    use super::lib::rc::Rc;
+    use crate::lib::iter::FromIterator;
+    use crate::lib::borrow::ToOwned;
+    use crate::lib::boxed::Box;
+    use crate::lib::rc::Rc;
 
     #[test]
     pub fn test_zero() {
@@ -1433,7 +1440,7 @@ mod test {
 
     #[test]
     fn test_borrow() {
-        use std::borrow::Borrow;
+        use lib::borrow::Borrow;
 
         let mut a: StackVec<[u32; 3]> = StackVec::new();
         a.push(1);
@@ -1446,7 +1453,7 @@ mod test {
 
     #[test]
     fn test_borrow_mut() {
-        use std::borrow::BorrowMut;
+        use lib::borrow::BorrowMut;
 
         let mut a: StackVec<[u32; 3]> = StackVec::new();
         a.push(1);
@@ -1504,7 +1511,6 @@ mod test {
     }
 
     #[test]
-    #[allow(deprecated)]
     fn veclike_deref_slice() {
         use super::VecLike;
 
